// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// This is a sample indicator plotting a SMA in three colors.
    /// </summary>
    [Description("Stochastics plotted with three colors.")]
    [Gui.Design.DisplayName("Stochastics_Color")]
    public class Stochastics_Color : Indicator
    {
        #region Variables
        private int                 periodD = 7;
        private int                 periodK = 14;
        private int                 smooth  = 3;
		private int iUpperLimit = 80;
		private int iLowerLimit = 20;
		private bool iUseAudioAlert = false;
		private string  iSoundAlertUp	   = "alert2.wav";
		private string  iSoundAlertDown	   = "alert2.wav";	
		
		private DataSeries sD;
        #endregion
        
        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            // Add one plot for every color you wish to use.
            Add(new Plot(new Pen(Color.DodgerBlue, 1), "K"));
            Add(new Plot(new Pen(Color.LimeGreen, 2), PlotStyle.Line, "Rising"));
            Add(new Plot(new Pen(Color.Red, 2), PlotStyle.Line, "Falling"));
            Add(new Plot(new Pen(Color.Yellow, 2), PlotStyle.Line, "Neutral"));
            
			Add(new Plot(new Pen(Color.YellowGreen, 2), PlotStyle.Line, "TopLimit"));
			Add(new Plot(new Pen(Color.DarkViolet, 2), PlotStyle.Line, "BottomLimit"));		
			
			sD = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            K.Set(Stochastics(PeriodD, PeriodK, Smooth).K[0]);
            // Checks to make sure we have at least 1 bar before continuing
            if (CurrentBar < 1)
                return;
			
			TopLimit.Set(iUpperLimit);
			BottomLimit.Set(iLowerLimit);
			
			sD.Set(Stochastics(PeriodD, PeriodK, Smooth).D[0]);
            
            // Plot green if the %D is rising
            // Rising() returns true when the current value is greater than the value of the previous bar.
            if (Rising(sD))
            {
                // Connects the rising plot segment with the other plots
                RisingPlot.Set(1, sD[1]);              
                
                // Adds the new rising plot line segment to the line
                RisingPlot.Set(sD[0]);             
            }
            
            // Plot red if the %D is falling
            // Falling() returns true when the current value is less than the value of the previous bar.
            else if (Falling(sD))
            {
                // Connects the new falling plot segment with the rest of the line
                FallingPlot.Set(1, sD[1]);
                
                // Adds the new falling plot line segment to the line
                FallingPlot.Set(sD[0]);
            }            
            // Plot yellow if the %D is neutral
            else
            {
                // Connects the neutral plot segment with the rest of the line
                NeutralPlot.Set(1, sD[1]);
                
                // Adds the new neutral plot line segment to the line
                NeutralPlot.Set(sD[0]);
            }
						
			//Audio Alerts
			if(!Historical && iUseAudioAlert)
			{
				if(CrossBelow(sD, iUpperLimit, 1))
					PlaySound(iSoundAlertDown);
				if(CrossAbove(sD, iLowerLimit, 1))
					PlaySound(iSoundAlertUp);				
			}
        }

        #region Properties
        
        /// <summary>
        /// Gets the fast K value.
        /// </summary>
        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries K
        {
            get { return Values[0]; }
        }
        
        // Adds three DataSeries to store the values of the three plots added in Initialize()
        [Browsable(false)]  // this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]       // this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RisingPlot
        {
            get { return Values[1]; }
        }

        [Browsable(false)]  // this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]       // this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries FallingPlot
        {
            get { return Values[2]; }
        }
        
        [Browsable(false)]  // this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]       // this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries NeutralPlot
        {
            get { return Values[3]; }
        }
		
        [Browsable(false)]  // this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]       // this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TopLimit
        {
            get { return Values[4]; }
        }
        
        [Browsable(false)]  // this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]       // this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries BottomLimit
        {
            get { return Values[5]; }
        }		
        
        // Adds an input for the period of the SMA
        /// <summary>
        /// </summary>
        [Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int PeriodK
        {
            get { return periodK; }
            set { periodK = Math.Max(1, value); }
        }
        
        // Adds an input for the period of the SMA
        /// <summary>
        /// </summary>
        [Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int PeriodD
        {
            get { return periodD; }
            set { periodD = Math.Max(1, value); }
        }
        
        [Description("Smoothing period for SMA")]
        [Category("Parameters")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }
		
        [Description("Upper Limit")]
        [Category("Parameters")]
        public int UpperLimit
        {
            get { return iUpperLimit; }
            set { iUpperLimit = Math.Max(0, value); }
        }
        [Description("Lower Limit")]
        [Category("Parameters")]
        public int LowerLimit
        {
            get { return iLowerLimit; }
            set { iLowerLimit = Math.Max(0, value); }
        }	
        [Description("")]
        [Category("Parameters")]
        public bool UseAudioAlert
        {
            get { return iUseAudioAlert; }
            set { iUseAudioAlert = value; }
        }	
        [Description("")]
        [Category("Parameters")]
        public string SoundAlertUp
        {
            get { return iSoundAlertUp; }
            set { iSoundAlertUp = value; }
        }				
        [Description("")]
        [Category("Parameters")]
        public string SoundAlertDown
        {
            get { return iSoundAlertDown; }
            set { iSoundAlertDown = value; }
        }			
		
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Stochastics_Color[] cacheStochastics_Color = null;

        private static Stochastics_Color checkStochastics_Color = new Stochastics_Color();

        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        public Stochastics_Color Stochastics_Color(int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            return Stochastics_Color(Input, lowerLimit, periodD, periodK, smooth, soundAlertDown, soundAlertUp, upperLimit, useAudioAlert);
        }

        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        public Stochastics_Color Stochastics_Color(Data.IDataSeries input, int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            checkStochastics_Color.LowerLimit = lowerLimit;
            lowerLimit = checkStochastics_Color.LowerLimit;
            checkStochastics_Color.PeriodD = periodD;
            periodD = checkStochastics_Color.PeriodD;
            checkStochastics_Color.PeriodK = periodK;
            periodK = checkStochastics_Color.PeriodK;
            checkStochastics_Color.Smooth = smooth;
            smooth = checkStochastics_Color.Smooth;
            checkStochastics_Color.SoundAlertDown = soundAlertDown;
            soundAlertDown = checkStochastics_Color.SoundAlertDown;
            checkStochastics_Color.SoundAlertUp = soundAlertUp;
            soundAlertUp = checkStochastics_Color.SoundAlertUp;
            checkStochastics_Color.UpperLimit = upperLimit;
            upperLimit = checkStochastics_Color.UpperLimit;
            checkStochastics_Color.UseAudioAlert = useAudioAlert;
            useAudioAlert = checkStochastics_Color.UseAudioAlert;

            if (cacheStochastics_Color != null)
                for (int idx = 0; idx < cacheStochastics_Color.Length; idx++)
                    if (cacheStochastics_Color[idx].LowerLimit == lowerLimit && cacheStochastics_Color[idx].PeriodD == periodD && cacheStochastics_Color[idx].PeriodK == periodK && cacheStochastics_Color[idx].Smooth == smooth && cacheStochastics_Color[idx].SoundAlertDown == soundAlertDown && cacheStochastics_Color[idx].SoundAlertUp == soundAlertUp && cacheStochastics_Color[idx].UpperLimit == upperLimit && cacheStochastics_Color[idx].UseAudioAlert == useAudioAlert && cacheStochastics_Color[idx].EqualsInput(input))
                        return cacheStochastics_Color[idx];

            Stochastics_Color indicator = new Stochastics_Color();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.LowerLimit = lowerLimit;
            indicator.PeriodD = periodD;
            indicator.PeriodK = periodK;
            indicator.Smooth = smooth;
            indicator.SoundAlertDown = soundAlertDown;
            indicator.SoundAlertUp = soundAlertUp;
            indicator.UpperLimit = upperLimit;
            indicator.UseAudioAlert = useAudioAlert;
            indicator.SetUp();

            Stochastics_Color[] tmp = new Stochastics_Color[cacheStochastics_Color == null ? 1 : cacheStochastics_Color.Length + 1];
            if (cacheStochastics_Color != null)
                cacheStochastics_Color.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheStochastics_Color = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Stochastics_Color Stochastics_Color(int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            return _indicator.Stochastics_Color(Input, lowerLimit, periodD, periodK, smooth, soundAlertDown, soundAlertUp, upperLimit, useAudioAlert);
        }

        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        public Indicator.Stochastics_Color Stochastics_Color(Data.IDataSeries input, int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            return _indicator.Stochastics_Color(input, lowerLimit, periodD, periodK, smooth, soundAlertDown, soundAlertUp, upperLimit, useAudioAlert);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Stochastics_Color Stochastics_Color(int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            return _indicator.Stochastics_Color(Input, lowerLimit, periodD, periodK, smooth, soundAlertDown, soundAlertUp, upperLimit, useAudioAlert);
        }

        /// <summary>
        /// Stochastics plotted with three colors.
        /// </summary>
        /// <returns></returns>
        public Indicator.Stochastics_Color Stochastics_Color(Data.IDataSeries input, int lowerLimit, int periodD, int periodK, int smooth, string soundAlertDown, string soundAlertUp, int upperLimit, bool useAudioAlert)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Stochastics_Color(input, lowerLimit, periodD, periodK, smooth, soundAlertDown, soundAlertUp, upperLimit, useAudioAlert);
        }

    }
}
#endregion
